/**
 * 
 */

var irpApplication;
var irpProject;
var properties;
var logger;
var oldProfile;
var newProfile;
var iRPInstance = Java.type('com.telelogic.rhapsody.core.IRPInstance');
var iRPDependency = Java.type('com.telelogic.rhapsody.core.IRPDependency');
var iRPGeneralization = Java.type('com.telelogic.rhapsody.core.IRPGeneralization');
var iRPHyperLink = Java.type('com.telelogic.rhapsody.core.IRPHyperLink');
var iRPObjectModelDiagram = Java.type('com.telelogic.rhapsody.core.IRPObjectModelDiagram');
var iRPPackage = Java.type('com.telelogic.rhapsody.core.IRPPackage');

setApplication = function (application){
	irpApplication = application;
}

setProject = function (project){
	irpProject = project;
}

setProperties = function (newProperties){
	properties = newProperties;
}

setOldProfile = function (profile){
	oldProfile = profile;
}

setNewProfile = function (profile){
	newProfile = profile;
}

setLogger = function (newLogger){
	logger = newLogger;
}

getProject = function (){
	return irpProject;
}

evaluateRule = function(mappingArray, stereotypeKey, element){
	logger.info("evaluate Migration Rule "+stereotypeKey+" on element "+element.getFullPathName());
	mappingArray[stereotypeKey](element);
}

//Migration policy

handleRenameStereotype = function(element, oldStereotypeName, newStereotypeName){
	unapplyStereotype(element, oldStereotypeName);
	applyStereotype(element, newStereotypeName);
}

handleRenameTag = function(element, oldTag, newTag){
	logger.fine("rename Tag "+oldTag+" to "+newTag+" in "+element.getFullPathName());
	var objectAsObjectType;
	if(iRPInstance.class.isInstance(element)){
        objectAsObjectType=(element.getObjectAsObjectType());
    }
	var tag;
	if(objectAsObjectType!=null){
		tag = getTag(objectAsObjectType, oldTag);
	}
	else{
		tag = getTag(element, oldTag);
	}
	if(tag!=null){
		var value = getTagValues(tag);
		logger.fine("move Tag value '"+value+"' on Tag "+newTag+" in "+element.getFullPathName());
		setTagValue(element, newTag, value);
		deleteElement(tag);
	}
	else{
		logger.fine("Tag not found "+tag.getName()+" in "+element.getFullPathName());
	}
}

handleDeleteStereotype = function(element, stereotypeName){
	unapplyStereotype(element, stereotypeName);
}

handleDeleteTag = function(element, tagName){
	removeTagFromOldProfile(element, tagName);
}

handleDeleteStereotypeFromAggregates = function(element, stereotypeName){
	if(!unapplyStereotypeFromCommentChild(element, stereotypeName)){
		unapplyDependencyFromCommentChild(element, stereotypeName);
	}
}

handleChangeGeneralizationTarget = function(element){
	//Keep old Tag from old Stereotype superType
}

handleChangeTargetPropertyOnStereotype = function(element, stereotypeName){
	unapplyStereotype(element, stereotypeName); 
}

handleModifyTagType = function(element, tagName, newType){
	var tag = getTag(element, tagName);
	tag.setType(getElementFromNewProfile(newType));
}
//////////////////////////////////

getTagValues = function(element){
	var values = element.getValueSpecifications();
    var stCount = values.getCount();
	var allValues;
	
    for (var i = 1; i <= stCount; i++) {
		var value = values.getItem(i);
		if(allValues==null){
			allValues = value.getValue();
		}
		else
			allValues = allValues+", "+value.getValue();
	}
	logger.fine("values of tag "+element.getName()+" : "+allValues+" ("+stCount+")");
	return allValues;
}

getElementFromOldProfile = function(name, metaClass){
	var element = oldProfile.findNestedElementRecursive(name, metaClass);
	return element;
}

getElementFromNewProfile = function(name, metaClass){
	var element = newProfile.findNestedElementRecursive(name, metaClass);
	return element;
}

getPropertyValue = function(propertyKey){
	var result = properties.getProperty(propertyKey, null);
	if(result==null){
		throw "The property " + propertyKey + " does not exist";
	}
	return result; 
}

getPropertyValueOfElement = function(element, ruleName){
	var guid = element.getGUID();
	var guidKey = guid.substring(5, guid.length());
	var key = ruleName+"|"+guidKey;
	var newStereotypeName = getPropertyValue(key);
	return newStereotypeName;
}

unapplyStereotype = function(element, stereotypeName){
	logger.fine("unapply Stereotype "+stereotypeName+" on "+element.getFullPathName());
	var stereotypes = element.getStereotypes();
    var stCount = stereotypes.getCount();
    for (var i = 1; i <= stCount; i++) {
		var stereotypeF = stereotypes.getItem(i);
        var name = stereotypeF.getName();
		if(name.equals(stereotypeName)){
			element.removeStereotype(stereotypeF);
			logger.fine("unapply Stereotype "+stereotypeName+" success on "+element.getFullPathName());
			return;
		}
	}
	logger.fine("unapply Stereotype "+stereotypeName+" failed on "+element.getFullPathName());
}

applyStereotype = function(element, stereotypeName){
	logger.fine("apply Stereotype "+stereotypeName+" on "+element.getFullPathName());
	if(stereotypeName!=null){
		var stereotypes = getProject().getAllStereotypes();
		var stCount = stereotypes.getCount();
		for (var i = 1; i <= stCount; i++) {
			var stereotypeF = stereotypes.getItem(i);
			var name = stereotypeF.getName();
			if(name.equals(stereotypeName)){
				element.addSpecificStereotype(stereotypeF);
				logger.fine("apply Stereotype "+stereotypeName+" success on "+element.getFullPathName());
				return;
			}
		}
	}
	logger.fine("apply Stereotype "+stereotypeName+" failed on "+element.getFullPathName());
}

getTag = function(element, tagName){
	var tag = element.getTag(tagName);
	if (tag!=null && !tag.getOwner().equals(element)) {
		try	{
			tag = element.setTagValue(tag, null);
		}
		catch(e){
			logger.fine("error, impossible to set null on tag "+e);
		}
    }
	if (tag === undefined || tag === null) {
		logger.fine("tag "+tagName+" not found in "+element.getFullPathName());
	}
	return tag;
}

setTagValue = function(element, tagName, tagValue){
	logger.fine("Try to set value '"+tagValue+"' to tag "+tagName+" in element "+element.getFullPathName());
	var tag = getTag(element, tagName);
	if ((tag != null) && (tagValue != null)) {
		logger.fine("set value '"+tagValue+"' to tag "+tagName+" in element "+element.getFullPathName());
		try{
			tag.addStringDefaultValue(tagValue.toString());
		}
		catch(e){
			logger.fine("error, impossible to set value '"+tagValue+"' to tag "+tagName+" in element "+element.getFullPathName()+" "+e);
		}
	}
	return tag;
}

getElementWithStereotype = function(element, stereotypeName){
	var elements = null;
	if(iRPPackage.class.isInstance(element)){
		elements = element.getAllNestedElements();
	}
	else{
		elements = element.getNestedElements();
	}
	var stCount = elements.getCount();
    for (var i = 1; i <= stCount; i++) {
        var child = elements.getItem(i);
		if(hasStereotypeApplied(child, stereotypeName)){
			return child;
		}
	}
	return null;
}

hasStereotypeApplied = function(element, stereotypeName){
	var stereotypes = element.getStereotypes();
    var stCount = stereotypes.getCount();
    for (var i = 1; i <= stCount; i++) {
		var stereotypeF = stereotypes.getItem(i);
        var name = stereotypeF.getName();
		if(name.equals(stereotypeName)){
			return true;
		}
	}
	return false;
}

createElementOn = function(owner, metaType, stereotypeName, name){
	logger.fine("create "+stereotypeName+" in "+owner.getName());
	var newElement = owner.addNewAggr(metaType, name);
	applyStereotype(newElement, stereotypeName);
	return newElement;
}

getOrCreateElementOn = function(owner, metaType, stereotypeName, name){
	try{
		var elements = null;
		if(iRPPackage.class.isInstance(owner)){
			elements = owner.getAllNestedElements();
		}
		else{
			elements = owner.getNestedElements();
		}
		var stCount = elements.getCount();
		var child = null;
		for (var i = 1; i <= stCount; i++) {
			var element = elements.getItem(i);
			if(element.getName().equals(name) && element.getMetaClass().equals(metaType)){
				child = elements.getItem(i);
			}
		}
		if(child==null || !hasStereotypeApplied(child, stereotypeName)){
			child = owner.addNewAggr(metaType, name);
			applyStereotype(child, stereotypeName);
		}
		return child;
	}
	catch(e){
		logger.fine("error when trying to get or create "+stereotypeName+" element : "+e);
	}
	return null;
}

getAnnotationIn = function(owner, stereotypeName){
	var annotations = owner.getAnnotations();
    var stCount = annotations.getCount();
    for (var i = 1; i <= stCount; i++) {
		var annotation = annotations.getItem(i);
		if(hasStereotypeApplied(annotation, stereotypeName)){
			return annotation;
		}
	}
	return null;
}

getDependencyIn = function(owner, stereotypeName){
	var dependencies = owner.getDependencies();
    var stCount = dependencies.getCount();
    for (var i = 1; i <= stCount; i++) {
		var dependency = dependencies.getItem(i);
		if(hasStereotypeApplied(dependency, stereotypeName)){
			return dependency;
		}
	}
	return null;
}

removeTag = function(element, tagName){
	var tag = getTag(element, tagName);
	if(tag!=null){
		logger.fine("delete Tag "+tagName+" in "+element.getFullPathName());
		deleteElement(tag);
	}
}

removeTagFromOldProfile = function(element, tagName){
	var tag = getTag(element, tagName);
	if(tag!=null && ((tag.getFromProfile()!=null && tag.getFromProfile().getName().equals(oldProfile.getName())) || tag.getOwner().equals(element))){
		deleteElement(tag);
	}
}

removeCommentIn = function(element, stereotypeName){
	var child = getAnnotationIn(element, stereotypeName)
	if(child!=null){
		deleteElement(child);
	}
}

unapplyStereotypeFromCommentChild = function(element, stereotypeName){
	logger.fine("try to unapply stereotype "+stereotypeName+" on Comment of element "+element.getFullPathName());
	var child = getAnnotationIn(element, stereotypeName)
	if(child!=null){
		unapplyStereotype(child, stereotypeName);
		return true;
	}
	return false;
}
	
removeDependencyIn = function(element, stereotypeName){
	var child = getDependencyIn(element, stereotypeName)
	if(child!=null){
		deleteElement(child);
	}
}

unapplyDependencyFromCommentChild = function(element, stereotypeName){
	logger.fine("try to unapply stereotype "+stereotypeName+" on Dependency of element "+element.getFullPathName());
	var child = getDependencyIn(element, stereotypeName)
	if(child!=null){
		unapplyStereotype(child, stereotypeName);
		return true;
	}
	return false;
}

moveAnnotationsFromTo = function(from, to){
	var annotations = from.getAnnotations();
    var stCount = annotations.getCount();
    for (var i = 1; i <= stCount; i++) {
		try{
			var annotation = annotations.getItem(i);
			annotation.setOwner(to);
			logger.fine("move "+annotation.getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName());
		}
		catch(e){
			logger.fine("error, impossible to move "+annotation.getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName()+" : "+e);
		}
	}
	return null;
}

moveNestedElementsFromTo = function(from, to){
	logger.fine("move nestedElements from "+from.getFullPathName()+" to "+to.getFullPathName()); 
	var elementsToMove = [];
	elementsToMove = returnNestedElements(from, elementsToMove, 0);
	var arrayLength = elementsToMove.length;
	for (var i = 0; i < arrayLength; i++) {
		try{
			if(elementsToMove[i]!=null){
				logger.fine("try to move "+elementsToMove[i].getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName());
				elementsToMove[i].setOwner(to);
				logger.fine("move "+elementsToMove[i].getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName()+" on place"+i);
			}
			else{
				logger.fine("impossible to move element because null element on place"+i);
			}
		}
		catch(e){
			logger.fine("error, impossible to move  from "+from.getFullPathName()+" in "+to.getFullPathName()+" : "+e);
		}
	}
	return null;
}


returnNestedElements = function(element, elementsToMove, place) {
    var nestedElements = element.getNestedElements();
    var count = nestedElements.getCount();
    var otherClass=null;
    if(iRPInstance.class.isInstance(element)){
        otherClass=(element.getObjectAsObjectType());
    }
    for (var i = 1; i <= count; i++) {
        var nested = nestedElements.getItem(i);
        if(!nested.equals(otherClass)){
            logger.fine("nestedElement found "+nested.getClass().getSimpleName() + ": "+nested.getFullPathName()+" place : "+place);
			elementsToMove[place]=nested;
			place++;
        }
    }
    if(otherClass!=null)
        return returnNestedElements(otherClass, elementsToMove, place++);
	return elementsToMove;
}

returnReferencesElements = function(element) {
	try{
		var references = element.getReferences();
		var elementsToMove = [];
		var count = references.getCount();
		for (var i = 1; i <= count; i++) {
			try{
				var reference = references.getItem(i);
				logger.fine("reference found "+reference.getName()+ ": "+reference.getFullPathName()+" place : "+i-1);
				elementsToMove[i-1]=reference;
			}
			catch(e){
				logger.fine("error during getting refs on"+element.getName()+" : "+e);
			}
		}
	}
	catch(e){
		logger.fine("error, impossible to get references from "+element.getFullPathName()+" : "+e);
	}
	return elementsToMove;
}

updateReferencesElementsFromTo = function(from, to){
	logger.fine("updating references from "+from.getFullPathName()+" to "+to.getFullPathName());
	try{
		var elementsToUpdate = [];
		elementsToUpdate = returnReferencesElements(from);
		var arrayLength = elementsToUpdate.length;
		for (var i = 0; i < arrayLength; i++) {
			if(elementsToUpdate[i]!=null){
				logger.fine("try to update ref "+elementsToUpdate[i].getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName());
				if(iRPDependency.class.isInstance(elementsToUpdate[i])){
					elementsToUpdate[i].setDependsOn(to);
				}
				else if(iRPGeneralization.class.isInstance(elementsToUpdate[i])){
					elementsToUpdate[i].setBaseClass(to);
				}
				else if(iRPHyperLink.class.isInstance(elementsToUpdate[i])){
					elementsToUpdate[i].setDependsOn(to);
				}
				else if(iRPObjectModelDiagram.class.isInstance(elementsToUpdate[i])){
					updateGraphicalElements(elementsToUpdate[i], from, to);
				}
			}
			else{
				logger.fine("impossible to update ref  because null element on place"+i);
			}
		}
	}
	catch(e){
		logger.fine("error, impossible to update ref from "+from.getFullPathName()+" in "+to.getFullPathName()+" : "+e);
	}
	logger.fine("end to update ref from "+from.getFullPathName()+" in "+to.getFullPathName());
	return null;
}


deleteElement = function(element){
	logger.fine("delete "+element.getFullPathName());
	element.deleteFromProject();
}

create = function(owner, metaType, name){
	logger.fine("create "+metaType+" in "+owner.getFullPathName()+" named "+name);
	return owner.addNewAggr(metaType, name);
}

changeMetaClassAndStereotype = function(element, metaClass, newStereotypeName){
	logger.fine("Change metaClass and stereotype on "+element.getName());
	var newElement = create(element.getOwner(), metaClass, element.getName());
	applyStereotype(newElement, newStereotypeName);
	moveNestedElementsFromTo(element, newElement);
	updateReferencesElementsFromTo(element, newElement);
	deleteElement(element);
}

updateGraphicalElements = function(diagram, from, to){
	logger.warning("Deleting element "+from.getName()+" will cause GraphNode or GraphEdge removing in diagram "+diagram.getFullPathName());
}
